@@ -3,7 +3,6 @@
 
 ((window) => {
   const {
-    ObjectDefineProperty,
     StringPrototypeReplace,
     TypeError,
     Promise,
@@ -55,18 +54,6 @@
     return promise;
   }
 
-  function immutableDefine(
-    o,
-    p,
-    value,
-  ) {
-    ObjectDefineProperty(o, p, {
-      value,
-      configurable: false,
-      writable: false,
-    });
-  }
-
   // Keep in sync with `fromFileUrl()` in `std/path/win32.ts`.
   function pathFromURLWin32(url) {
     let p = StringPrototypeReplace(
@@ -164,7 +151,6 @@
     createResolvable,
     assert,
     AssertionError,
-    immutableDefine,
     pathFromURL,
     writable,
     nonEnumerable,
