@@ -75,7 +75,7 @@
   /**
    * @param {string} permission
    * @return {(boolean | string[])}
-   * */
+   */
   function parseArrayPermission(
     value,
     permission,
@@ -139,7 +139,13 @@
   class Worker extends EventTarget {
     #id = 0;
     #name = "";
-    #terminated = false;
+
+    // "RUNNING" | "CLOSED" | "TERMINATED"
+    // "TERMINATED" means that any controls or messages received will be
+    // discarded. "CLOSED" means that we have received a control
+    // indicating that the worker is no longer running, but there might
+    // still be messages left to receive.
+    #status = "RUNNING";
 
     constructor(specifier, options = {}) {
       super();
@@ -243,17 +249,17 @@
     }
 
     #pollControl = async () => {
-      while (!this.#terminated) {
+      while (this.#status === "RUNNING") {
         const [type, data] = await hostRecvCtrl(this.#id);
 
         // If terminate was called then we ignore all messages
-        if (this.#terminated) {
+        if (this.#status === "TERMINATED") {
           return;
         }
 
         switch (type) {
           case 1: { // TerminalError
-            this.#terminated = true;
+            this.#status = "CLOSED";
           } /* falls through */
           case 2: { // Error
             if (!this.#handleError(data)) {
@@ -270,7 +276,7 @@
           }
           case 3: { // Close
             log(`Host got "close" message from worker: ${this.#name}`);
-            this.#terminated = true;
+            this.#status = "CLOSED";
             return;
           }
           default: {
@@ -281,14 +287,16 @@
     };
 
     #pollMessages = async () => {
-      while (!this.terminated) {
+      while (this.#status !== "TERMINATED") {
         const data = await hostRecvMessage(this.#id);
-        if (data === null) break;
-        let message, transfer;
+        if (this.#status === "TERMINATED" || data === null) {
+          return;
+        }
+        let message, transferables;
         try {
           const v = deserializeJsMessageData(data);
           message = v[0];
-          transfer = v[1];
+          transferables = v[1];
         } catch (err) {
           const event = new MessageEvent("messageerror", {
             cancelable: false,
@@ -300,7 +308,7 @@
         const event = new MessageEvent("message", {
           cancelable: false,
           data: message,
-          ports: transfer,
+          ports: transferables.filter((t) => t instanceof MessagePort),
         });
         this.dispatchEvent(event);
       }
@@ -332,13 +340,14 @@
       }
       const { transfer } = options;
       const data = serializeJsMessageData(message, transfer);
-      if (this.#terminated) return;
-      hostPostMessage(this.#id, data);
+      if (this.#status === "RUNNING") {
+        hostPostMessage(this.#id, data);
+      }
     }
 
     terminate() {
-      if (!this.#terminated) {
-        this.#terminated = true;
+      if (this.#status !== "TERMINATED") {
+        this.#status = "TERMINATED";
         hostTerminateWorker(this.#id);
       }
     }
